@@ -1,6 +1,6 @@
 /*
     orzcharset
-    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -50,11 +50,6 @@ struct _OrzCharset
 #endif /* not GTK_CHECK_VERSION(2,4,0) */
 
   /*< public >*/
-
-  /*< private >*/
-#if GTK_CHECK_VERSION(2,4,0)
-  GtkListStore *store;
-#endif /* GTK_CHECK_VERSION(2,4,0) */
 };
 
 struct _OrzCharsetClass
