@@ -16,8 +16,8 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#include "misc/fileio.h"
 #include "orzmdi.h"
+#include "misc/fileio.h"
 
 
 static void orz_mdi_class_init (OrzMdiClass *klass);
@@ -272,6 +272,36 @@ orz_mdi_get_type (void)
 }
 
 
+/* ja:子ウィジェットが破棄されるとき */
+static void
+orz_mdi_remove (GtkContainer *container,
+                GtkWidget    *widget)
+{
+  GtkWidget *tab_label;
+
+  tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (container), widget);
+  if (tab_label)
+    {
+      OrzMdiTab *tab;
+
+      tab = g_object_get_data (G_OBJECT (tab_label), "user_data");
+      if (tab)
+        {
+          g_signal_emit (container,
+                    orz_mdi_signals[REMOVE_WINDOW_SIGNAL],
+                    0,
+                    gtk_notebook_page_num (GTK_NOTEBOOK (container), widget),
+                    tab->page_data);
+          file_remove_edit (ORZ_MDI (container), tab, NULL);
+          g_free (tab->file);
+          g_free (tab);
+          g_object_set_data (G_OBJECT (tab_label), "user_data", NULL);
+        }
+    }
+  ((GtkContainerClass *)parent_class)->remove (container, widget);
+}
+
+
 static void
 orz_mdi_switch_page (GtkNotebook     *notebook,
                      GtkNotebookPage *page,
@@ -308,15 +338,18 @@ orz_mdi_class_init (OrzMdiClass *klass)
   GObjectClass *gobject_class;
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
+  GtkContainerClass *container_class;
   GtkNotebookClass *notebook_class;
 
-  gobject_class = (GObjectClass*) klass;
-  object_class = (GtkObjectClass*) klass;
-  widget_class = (GtkWidgetClass*) klass;
-  notebook_class = (GtkNotebookClass*) klass;
+  gobject_class = (GObjectClass *) klass;
+  object_class = (GtkObjectClass *) klass;
+  widget_class = (GtkWidgetClass *) klass;
+  container_class = (GtkContainerClass *) klass;
+  notebook_class = (GtkNotebookClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
+  container_class->remove = orz_mdi_remove;
   notebook_class->switch_page = orz_mdi_switch_page;
 
   orz_mdi_signals[ADD_WINDOW_SIGNAL]
@@ -450,7 +483,7 @@ orz_mdi_destroy (GtkWidget *widget,
 
 
 /* ja:ドロップされたとき */
-void
+static void
 orz_mdi_tab_drag_begin (GtkWidget      *widget,
                         GdkDragContext *context,
                         gpointer        user_data)
@@ -689,6 +722,107 @@ orz_mdi_create_tab_label (OrzMdi         *mdi,
 }
 
 
+/*  ja:ページを最後に追加する
+          mdi,ウィジェット
+        child,内容
+         file,ファイル名
+      created,TRUE:新規作成,FALSE:既存ファイル
+    page_data,ページデータ
+          RET,ページ(0...,-1:エラー)                                        */
+gint
+orz_mdi_append_page (OrzMdi         *mdi,
+                     GtkWidget      *child,
+                     const gchar    *file,
+                     const gboolean  created,
+                     gpointer        page_data)
+{
+  return orz_mdi_insert_page_menu (mdi, child, NULL, -1,
+                                                    file, created, page_data);
+}
+
+
+/*  ja:ページを最後に追加する
+          mdi,ウィジェット
+        child,内容
+         menu,メニュー
+         file,ファイル名
+      created,TRUE:新規作成,FALSE:既存ファイル
+    page_data,ページデータ
+          RET,ページ(0...,-1:エラー)                                        */
+gint
+orz_mdi_append_page_menu (OrzMdi         *mdi,
+                          GtkWidget      *child,
+                          GtkWidget      *menu,
+                          const gchar    *file,
+                          const gboolean  created,
+                          gpointer        page_data)
+{
+  return orz_mdi_insert_page_menu (mdi, child, menu, -1,
+                                                    file, created, page_data);
+}
+
+
+/*  ja:ページを最初に追加する
+          mdi,ウィジェット
+        child,内容
+         file,ファイル名
+      created,TRUE:新規作成,FALSE:既存ファイル
+    page_data,ページデータ
+          RET,ページ(0...,-1:エラー)                                        */
+gint
+orz_mdi_prepend_page (OrzMdi         *mdi,
+                      GtkWidget      *child,
+                      const gchar    *file,
+                      const gboolean  created,
+                      gpointer        page_data)
+{
+  return orz_mdi_insert_page_menu (mdi, child, NULL, 0,
+                                                    file, created, page_data);
+}
+
+
+/*  ja:ページを最初に追加する
+          mdi,ウィジェット
+        child,内容
+         menu,メニュー
+         file,ファイル名
+      created,TRUE:新規作成,FALSE:既存ファイル
+    page_data,ページデータ
+          RET,ページ(0...,-1:エラー)                                        */
+gint
+orz_mdi_prepend_page_menu (OrzMdi         *mdi,
+                           GtkWidget      *child,
+                           GtkWidget      *menu,
+                           const gchar    *file,
+                           const gboolean  created,
+                           gpointer        page_data)
+{
+  return orz_mdi_insert_page_menu (mdi, child, menu, 0,
+                                                    file, created, page_data);
+}
+
+
+/*  ja:ページを挿入する
+          mdi,ウィジェット
+        child,内容
+     position,位置(0...)
+         file,ファイル名
+      created,TRUE:新規作成,FALSE:既存ファイル
+    page_data,ページデータ
+       RET,ページ(0...,-1:エラー)                                           */
+gint
+orz_mdi_insert_page (OrzMdi         *mdi,
+                     GtkWidget      *child,
+                     const gint      position,
+                     const gchar    *file,
+                     const gboolean  created,
+                     gpointer        page_data)
+{
+  return orz_mdi_insert_page_menu (mdi, child, NULL, position,
+                                                    file, created, page_data);
+}
+
+
 /*  ja:ページを挿入する
           mdi,ウィジェット
         child,内容
@@ -725,13 +859,15 @@ orz_mdi_insert_page_menu (OrzMdi         *mdi,
 }
 
 
-#if !GTK_CHECK_VERSION(2,2,0)
 /*  ja:ページ数を取得する
     mdi,ウィジェット
     RET,ページ数                                                            */
 gint
 orz_mdi_get_n_pages (OrzMdi *mdi)
 {
+#if GTK_CHECK_VERSION(2,2,0)
+  return gtk_notebook_get_n_pages (GTK_NOTEBOOK (mdi));
+#else /* not GTK_CHECK_VERSION(2,2,0) */
   gint n_pages = 0;
 
   if (mdi)
@@ -746,8 +882,8 @@ orz_mdi_get_n_pages (OrzMdi *mdi)
         }
     }
   return n_pages;
-}
 #endif /* not GTK_CHECK_VERSION(2,2,0) */
+}
 
 
 /*  ja:ページをデータから取得する
