@@ -16,8 +16,11 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#include <locale.h>
 #include "tmaid.h"
+#include <locale.h>
+#ifdef G_OS_WIN32
+# include <windows.h>
+#endif /* G_OS_WIN32 */
 #include "command.h"
 #include "file.h"
 #include "find.h"
@@ -63,13 +66,11 @@ GdkAtom atom_targets;               /* ja:ターゲット */
 GdkAtom atom_text;                  /* ja:テキスト */
 GdkAtom atom_compound;              /* ja:コンパウンド */
 GdkAtom atom_utf8;                  /* ja:UTF-8 */
+GtkPositionType n_pos = GTK_POS_TOP;/* ja:ノートブックの位置 */
 GtkWidget *window;                  /* ja:ウインドウ */
 GtkWidget *mdi;                     /* en:MDI */
 GtkWidget *status;                  /* ja:ステータスウインドウ */
 GtkWidget *label_cursor;            /* ja:カーソル座標 */
-GtkPositionType n_pos = GTK_POS_TOP;/* ja:ノートブックの位置 */
-GtkItemFactory *ifactory_menu;      /* ja:メニュー */
-GtkItemFactory *ifactory_popup;     /* ja:ポップアップ */
 PangoFontDescription *system_font = NULL;
 GdkColor system_color[12] = {
     {0, 0x0000, 0x0000, 0x0000}, {0, 0xffff, 0xffff, 0xffff},
@@ -193,8 +194,7 @@ main (int   argc,
       char *argv[])
 {
   gboolean *arg_files, success[12];
-  gchar *group_name, *key, *tmp, *utf8str, *data, *user_config = NULL;
-  const gchar *charset;
+  gchar *user_config = NULL;
 #if ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32)
   LPITEMIDLIST pidl = NULL;
 #endif /* ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
@@ -206,7 +206,6 @@ main (int   argc,
 #endif /* not LOCALEDIR */
   gsize length;
   gint i, count = 0, init_line = 0, license = -1;
-  TmaidWindow *tmaid;
   GKeyFile *key_file;
   GList *glist;
   GObject *accel, *winmenu;
@@ -270,7 +269,7 @@ main (int   argc,
       if (SHGetPathFromIDList (pidl, szPath))
 #  ifdef UNICODE
         {
-          gchar *dir;
+          gchar *dir, *utf8str;
 
           utf8str = g_utf16_to_utf8 (szPath, -1, NULL, NULL, NULL);
           dir = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
@@ -296,6 +295,8 @@ main (int   argc,
   key_file = g_key_file_new ();
   if (g_key_file_load_from_file (key_file, user_config, G_KEY_FILE_NONE, NULL))
     {
+      gchar *utf8str;
+
       license = g_key_file_get_integer (key_file, "tmaid", "license", NULL);
       orz_history_set_num (ORZ_HISTORY (history),
                 g_key_file_get_integer (key_file, "tmaid", "history", NULL));
@@ -321,6 +322,8 @@ main (int   argc,
       /* ja:ファイルの履歴 */
       for (i = orz_history_get_num (ORZ_HISTORY (history)) - 1; i >= 0; i--)
         {
+          gchar *key;
+
           key = g_strdup_printf ("history%d", i);
           utf8str = g_key_file_get_string (key_file, "tmaid", key, NULL);
           g_free (key);
@@ -342,6 +345,8 @@ main (int   argc,
       find_num = g_key_file_get_integer (key_file, "tmaid", "find_num", NULL);
       for (i = 0; i < find_num; i++)
         {
+          gchar *key;
+
           key = g_strdup_printf ("find%02d", i);
           find_text[i] = g_key_file_get_string (key_file, "tmaid", key, NULL);
           g_free (key);
@@ -358,6 +363,8 @@ main (int   argc,
                                                                         NULL);
       for (i = 0; i < replace_num; i++)
         {
+          gchar *key;
+
           key = g_strdup_printf ("replace%02d", i);
           replace_text[i] = g_key_file_get_string (key_file, "tmaid", key,
                                                                         NULL);
@@ -410,6 +417,7 @@ main (int   argc,
         ftype = g_malloc (ftnum * sizeof (FileType));
       for (i = 0; i < ftnum; i++)
         {
+          gchar *group_name;
           gint *list;
 
           group_name = g_strdup_printf ("filetype%04d", i);
@@ -469,14 +477,20 @@ main (int   argc,
                                                 group_name, "charset", NULL);
           if (ftype[i].charset)
             {
+              gchar *tmp;
+
               tmp = charuty_strip (ftype[i].charset);
               g_free (ftype[i].charset);
               ftype[i].charset = tmp;
             }
           if (!ftype[i].charset)
-            ftype[i].charset = g_get_charset (&charset)
+            {
+              const gchar *charset;
+
+              ftype[i].charset = g_get_charset (&charset)
                                         ? g_strdup_printf ("%s,UTF-8", charset)
                                                         : g_strdup ("UTF-8");
+            }
           /* ja:フォント */
           ftype[i].font_name = g_key_file_get_string (key_file,
                                                     group_name, "font", NULL);
@@ -572,6 +586,7 @@ TMAID_COPYRIGHT_NOTICE"\n"
     orz_history_set_num (ORZ_HISTORY (history), arg_h.val);
     if (arg_l.val >= 0)
       init_line = arg_l.val;
+    newfile = arg_n.flag;
     def_width = array[0];
     def_height = array[1];
     second = arg_x.flag;
@@ -580,6 +595,8 @@ TMAID_COPYRIGHT_NOTICE"\n"
   /* ja:ファイルタイプ */
   if (ftnum <= 0)
     {
+      const gchar *charset;
+
       ftnum = 1;
       ftype = g_malloc0 (ftnum * sizeof (FileType));
       ftype[0].text = g_strdup (_("All Files"));
@@ -591,7 +608,7 @@ TMAID_COPYRIGHT_NOTICE"\n"
       ftype[0].syscol = TRUE;
       /* ja:キャラクターセット */
       ftype[0].charset = !g_get_charset (&charset) && charset
-                ? g_strdup_printf ("%s,UTF-8", charset) : g_strdup ("UTF-8");
+                ? g_strconcat (charset, ",UTF-8", NULL) : g_strdup ("UTF-8");
       /* ja:色 */
       g_memmove (ftype[0].color, system_color, sizeof (GdkColor) * 12);
       gdk_colormap_alloc_colors (gdk_colormap_get_system (),
@@ -655,8 +672,15 @@ TMAID_COPYRIGHT_NOTICE"\n"
   accel_group = gtk_accel_group_new ();
   misc_create_menu (menu_entries, accel_group);
   /* ja:ファイルの履歴 */
+  for (i = 0; menu_entries[i].path; i++)
+#if GLIB_CHECK_VERSION(2,2,0)
+    if (g_str_has_prefix (menu_entries[i].path, "/file/"))
+#else /* not GLIB_CHECK_VERSION(2,2,0) */
+    if (g_strncmp (menu_entries[i].path, "/file/", 6) == 0)
+#endif /* not GLIB_CHECK_VERSION(2,2,0) */
+      count++;
   orz_history_set_menu (ORZ_HISTORY (history),
-                        misc_find_menu (menu_entries, "/file/"), MENUFILE - 2);
+                        misc_find_menu (menu_entries, "/file/"), count - 3);
   g_signal_connect (ORZ_HISTORY (history), "activate",
                             G_CALLBACK (signal_activate_menu_history), NULL);
   /* ja:ウインドウメニュー */
@@ -746,9 +770,12 @@ TMAID_COPYRIGHT_NOTICE"\n"
                                     (gtk_widget_get_pango_context (label)));
 
   /* ja:新規ウインドウ */
+  count = 0;
   for (i = 1; i < argc; i++)
     if (arg_files[i])
       {
+        TmaidWindow *tmaid;
+
         tmaid = file_open_edit (argv[i]);
         if (init_line != 0)/* ja:指定行に移動 */
           jump_operation (tmaid, init_line);
@@ -761,186 +788,215 @@ TMAID_COPYRIGHT_NOTICE"\n"
   gtk_main ();
 
   /* ja:レジストリに書き込む */
-  key_file = g_key_file_new ();
-  g_key_file_load_from_file (key_file, user_config, G_KEY_FILE_NONE, NULL);
-  g_key_file_set_integer (key_file, "tmaid", "license", license);
-  g_key_file_set_integer (key_file, "tmaid", "history",
+  if (user_config)
+    {
+      gsize length;
+      gchar *data, *group_name, *utf8str;
+
+      key_file = g_key_file_new ();
+      g_key_file_load_from_file (key_file, user_config, G_KEY_FILE_NONE, NULL);
+      g_key_file_set_integer (key_file, "tmaid", "license", license);
+      g_key_file_set_integer (key_file, "tmaid", "history",
                                 orz_history_get_num (ORZ_HISTORY (history)));
-  g_key_file_set_boolean (key_file, "tmaid", "newfile", newfile);
-  g_key_file_set_boolean (key_file, "tmaid", "second", second);
-  g_key_file_set_integer (key_file, "tmaid", "def_width", def_width);
-  g_key_file_set_integer (key_file, "tmaid", "def_height", def_height);
-  g_key_file_set_integer (key_file, "tmaid", "n_pos", n_pos);
-  g_key_file_set_integer (key_file, "tmaid", "ftnum", ftnum);
+      g_key_file_set_boolean (key_file, "tmaid", "newfile", newfile);
+      g_key_file_set_boolean (key_file, "tmaid", "second", second);
+      g_key_file_set_integer (key_file, "tmaid", "def_width", def_width);
+      g_key_file_set_integer (key_file, "tmaid", "def_height", def_height);
+      g_key_file_set_integer (key_file, "tmaid", "n_pos", n_pos);
+      g_key_file_set_integer (key_file, "tmaid", "ftnum", ftnum);
 #if GTK_CHECK_VERSION(2,4,0) || defined (G_OS_WIN32)
-  for (i = ftnum - 1; i >= 0; i--)
-    if (ftype[i].ft_id == findex)
-      break;
-  g_key_file_set_integer (key_file, "tmaid", "findex", i);
+      for (i = ftnum - 1; i >= 0; i--)
+        if (ftype[i].ft_id == findex)
+          break;
+      g_key_file_set_integer (key_file, "tmaid", "findex", i);
 #endif /* GTK_CHECK_VERSION(2,4,0) || defined (G_OS_WIN32) */
-  if (open_path)
-    {
-      utf8str = g_filename_to_utf8 (open_path, -1, NULL, NULL, NULL);
-      if (utf8str)
+      if (open_path)
         {
-          g_key_file_set_string (key_file, "tmaid", "openpath", utf8str);
-          g_free (utf8str);
+          utf8str = g_filename_to_utf8 (open_path, -1, NULL, NULL, NULL);
+          if (utf8str)
+            {
+              g_key_file_set_string (key_file, "tmaid", "openpath", utf8str);
+              g_free (utf8str);
+            }
         }
-    }
-  if (save_path)
-    {
-      utf8str = g_filename_to_utf8 (save_path, -1, NULL, NULL, NULL);
-      if (utf8str)
+      if (save_path)
         {
-          g_key_file_set_string (key_file, "tmaid", "savepath", utf8str);
-          g_free (utf8str);
+          utf8str = g_filename_to_utf8 (save_path, -1, NULL, NULL, NULL);
+          if (utf8str)
+            {
+              g_key_file_set_string (key_file, "tmaid", "savepath", utf8str);
+              g_free (utf8str);
+            }
         }
-    }
-  /* ja:ファイルの履歴 */
-  glist = orz_history_get_file (ORZ_HISTORY (history));
-  for (i = 0; i < g_list_length (glist); i++)
-    {
-      key = g_strdup_printf ("history%d", i);
-      utf8str = g_filename_to_utf8 (g_list_nth_data (glist, i), -1,
+      /* ja:ファイルの履歴 */
+      glist = orz_history_get_file (ORZ_HISTORY (history));
+      for (i = 0; i < g_list_length (glist); i++)
+        {
+          gchar *key;
+
+          key = g_strdup_printf ("history%d", i);
+          utf8str = g_filename_to_utf8 (g_list_nth_data (glist, i), -1,
                                                             NULL, NULL, NULL);
+          if (utf8str)
+            {
+              g_key_file_set_string (key_file, "tmaid", key, utf8str);
+              g_free (utf8str);
+            }
+          g_free (key);
+        }
+      g_list_free (glist);
+      while (i <= ORZ_HISTORY_NUM_MAX)
+        {
+          gchar *key;
+
+          key = g_strdup_printf ("history%d", i++);
+          g_key_file_remove_key (key_file, "tmaid", key, NULL);
+          g_free (key);
+        }
+      /* ja:検索 */
+      g_key_file_set_boolean (key_file, "tmaid", "find_arrow", find_arrow);
+      g_key_file_set_boolean (key_file, "tmaid", "find_ignorecase",
+                                                            find_ignorecase);
+      g_key_file_set_integer (key_file, "tmaid", "find_num", find_num);
+      for (i = 0; i < find_num; i++)
+        {
+          gchar *key;
+
+          key = g_strdup_printf ("find%02d", i);
+          g_key_file_set_string (key_file, "tmaid", key, find_text[i]);
+          g_free (key);
+        }
+      while (i < 32)
+        {
+          gchar *key;
+
+          key = g_strdup_printf ("find%02d", i++);
+          g_key_file_set_string (key_file, "tmaid", key, "");
+          g_free (key);
+        }
+      /* ja:置換 */
+      g_key_file_set_boolean (key_file, "tmaid", "replace_arrow",
+                                                                replace_arrow);
+      g_key_file_set_boolean (key_file, "tmaid", "replace_ignorecase",
+                                                        replace_ignorecase);
+      g_key_file_set_integer (key_file, "tmaid", "replace_num", replace_num);
+      for (i = 0; i < replace_num; i++)
+        {
+          gchar *key;
+
+          key = g_strdup_printf ("replace%02d", i);
+          g_key_file_set_string (key_file, "tmaid", key, replace_text[i]);
+          g_free (key);
+        }
+      while (i < 32)
+        {
+          gchar *key;
+
+          key = g_strdup_printf ("replace%02d", i++);
+          g_key_file_set_string (key_file, "tmaid", key, "");
+          g_free (key);
+        }
+      /* ja:印刷 */
+#ifdef G_OS_UNIX
+      g_key_file_set_boolean (key_file, "printer", "mode", printer.mode);
+      utf8str = g_filename_to_utf8 (printer.printer, -1, NULL, NULL, NULL);
       if (utf8str)
         {
-          g_key_file_set_string (key_file, "tmaid", key, utf8str);
+          g_key_file_set_string (key_file, "printer", "printer", utf8str);
           g_free (utf8str);
         }
-      g_free (key);
-    }
-  g_list_free (glist);
-  while (i <= ORZ_HISTORY_NUM_MAX)
-    {
-      key = g_strdup_printf ("history%d", i++);
-      g_key_file_remove_key (key_file, "tmaid", key, NULL);
-      g_free (key);
-    }
-  /* ja:検索 */
-  g_key_file_set_boolean (key_file, "tmaid", "find_arrow", find_arrow);
-  g_key_file_set_boolean (key_file, "tmaid", "find_ignorecase",
-                                                            find_ignorecase);
-  g_key_file_set_integer (key_file, "tmaid", "find_num", find_num);
-  for (i = 0; i < find_num; i++)
-    {
-      key = g_strdup_printf ("find%02d", i);
-      g_key_file_set_string (key_file, "tmaid", key, find_text[i]);
-      g_free (key);
-    }
-  while (i < 32)
-    {
-      key = g_strdup_printf ("find%02d", i++);
-      g_key_file_set_string (key_file, "tmaid", key, "");
-      g_free (key);
-    }
-  /* ja:置換 */
-  g_key_file_set_boolean (key_file, "tmaid", "replace_arrow", replace_arrow);
-  g_key_file_set_boolean (key_file, "tmaid", "replace_ignorecase",
-                                                        replace_ignorecase);
-  g_key_file_set_integer (key_file, "tmaid", "replace_num", replace_num);
-  for (i = 0; i < replace_num; i++)
-    {
-      key = g_strdup_printf ("replace%02d", i);
-      g_key_file_set_string (key_file, "tmaid", key, replace_text[i]);
-      g_free (key);
-    }
-  while (i < 32)
-    {
-      key = g_strdup_printf ("replace%02d", i++);
-      g_key_file_set_string (key_file, "tmaid", key, "");
-      g_free (key);
-    }
-  /* ja:印刷 */
-#ifdef G_OS_UNIX
-  g_key_file_set_boolean (key_file, "printer", "mode", printer.mode);
-  utf8str = g_filename_to_utf8 (printer.printer, -1, NULL, NULL, NULL);
-  if (utf8str)
-    {
-      g_key_file_set_string (key_file, "printer", "printer", utf8str);
-      g_free (utf8str);
-    }
 #endif /* not G_OS_UNIX */
 #if defined (G_OS_UNIX) || defined (G_OS_WIN32)
-  g_key_file_set_integer (key_file, "printer", "copies", printer.copies);
+      g_key_file_set_integer (key_file, "printer", "copies", printer.copies);
 #endif /* defined (G_OS_UNIX) || defined (G_OS_WIN32) */
 #ifndef G_OS_WIN32
-  utf8str = g_filename_to_utf8 (printer.file, -1, NULL, NULL, NULL);
-  if (utf8str)
-    {
-      g_key_file_set_string (key_file, "printer", "file", utf8str);
-      g_free (utf8str);
-    }
-  g_key_file_set_integer (key_file, "printer", "width", printer.width);
-  g_key_file_set_integer (key_file, "printer", "height", printer.height);
-  g_key_file_set_integer (key_file, "printer", "left", printer.left);
-  g_key_file_set_integer (key_file, "printer", "top", printer.top);
-  g_key_file_set_integer (key_file, "printer", "right", printer.right);
-  g_key_file_set_integer (key_file, "printer", "bottom", printer.bottom);
+      utf8str = g_filename_to_utf8 (printer.file, -1, NULL, NULL, NULL);
+      if (utf8str)
+        {
+          g_key_file_set_string (key_file, "printer", "file", utf8str);
+          g_free (utf8str);
+        }
+      g_key_file_set_integer (key_file, "printer", "width", printer.width);
+      g_key_file_set_integer (key_file, "printer", "height", printer.height);
+      g_key_file_set_integer (key_file, "printer", "left", printer.left);
+      g_key_file_set_integer (key_file, "printer", "top", printer.top);
+      g_key_file_set_integer (key_file, "printer", "right", printer.right);
+      g_key_file_set_integer (key_file, "printer", "bottom", printer.bottom);
 #endif /* not G_OS_WIN32 */
-  /* ja:ファイルタイプ */
-  for (i = 0; i < ftnum; i++)
-    {
-      gint list[(sizeof (GdkColor) * 12 + sizeof (gint) - 1) / sizeof (gint)];
+      /* ja:ファイルタイプ */
+      for (i = 0; i < ftnum; i++)
+        {
+          gint list[(sizeof (GdkColor) * 12 + sizeof (gint) - 1)
+                                                            / sizeof (gint)];
 
-      group_name = g_strdup_printf ("filetype%04d", i);
-      g_key_file_set_string (key_file, group_name, "text", ftype[i].text);
-      g_key_file_set_string (key_file, group_name, "ext", ftype[i].ext);
-      g_key_file_set_integer (key_file, group_name, "crlf", ftype[i].crlf);
-      g_key_file_set_integer (key_file, group_name, "margin", ftype[i].margin);
-      g_key_file_set_integer (key_file, group_name, "tab", ftype[i].tab);
-      g_key_file_set_integer (key_file, group_name, "writemode",
+          group_name = g_strdup_printf ("filetype%04d", i);
+          g_key_file_set_string (key_file, group_name, "text", ftype[i].text);
+          g_key_file_set_string (key_file, group_name, "ext", ftype[i].ext);
+          g_key_file_set_integer (key_file, group_name, "crlf", ftype[i].crlf);
+          g_key_file_set_integer (key_file, group_name, "margin",
+                                                            ftype[i].margin);
+          g_key_file_set_integer (key_file, group_name, "tab", ftype[i].tab);
+          g_key_file_set_integer (key_file, group_name, "writemode",
                                                         ftype[i].writemode);
-      g_key_file_set_boolean (key_file, group_name, "associate",
+          g_key_file_set_boolean (key_file, group_name, "associate",
                                                         ftype[i].associate);
-      g_key_file_set_boolean (key_file, group_name, "autoindent",
+          g_key_file_set_boolean (key_file, group_name, "autoindent",
                                                         ftype[i].autoindent);
-      g_key_file_set_boolean (key_file, group_name, "code", ftype[i].code);
-      g_key_file_set_boolean (key_file, group_name, "eof", ftype[i].eof);
-      g_key_file_set_boolean (key_file, group_name, "hexcase",
+          g_key_file_set_boolean (key_file, group_name, "code", ftype[i].code);
+          g_key_file_set_boolean (key_file, group_name, "eof", ftype[i].eof);
+          g_key_file_set_boolean (key_file, group_name, "hexcase",
                                                             ftype[i].hexcase);
-      g_key_file_set_boolean (key_file, group_name, "html", ftype[i].html);
-      g_key_file_set_boolean (key_file, group_name, "esc", ftype[i].esc);
-      g_key_file_set_boolean (key_file, group_name, "limit", ftype[i].limit);
-      g_key_file_set_boolean (key_file, group_name, "overwrite",
+          g_key_file_set_boolean (key_file, group_name, "html", ftype[i].html);
+          g_key_file_set_boolean (key_file, group_name, "esc", ftype[i].esc);
+          g_key_file_set_boolean (key_file, group_name, "limit",
+                                                            ftype[i].limit);
+          g_key_file_set_boolean (key_file, group_name, "overwrite",
                                                         ftype[i].overwrite);
-      g_key_file_set_boolean (key_file, group_name, "space", ftype[i].space);
-      g_key_file_set_boolean (key_file, group_name, "syscol", ftype[i].syscol);
-      g_key_file_set_boolean (key_file, group_name, "tabconv", 
+          g_key_file_set_boolean (key_file, group_name, "space",
+                                                            ftype[i].space);
+          g_key_file_set_boolean (key_file, group_name, "syscol",
+                                                            ftype[i].syscol);
+          g_key_file_set_boolean (key_file, group_name, "tabconv",
                                                             ftype[i].tabconv);
-      g_key_file_set_boolean (key_file, group_name, "gline", ftype[i].gline);
-      g_key_file_set_boolean (key_file, group_name, "mline", ftype[i].mline);
-      g_key_file_set_boolean (key_file, group_name, "nline", ftype[i].nline);
-      g_key_file_set_boolean (key_file, group_name, "uline", ftype[i].uline);
-      g_key_file_set_boolean (key_file, group_name, "vline", ftype[i].vline);
-      /* ja:キャラクターセット */
-      g_key_file_set_string (key_file, group_name, "charset",
+          g_key_file_set_boolean (key_file, group_name, "gline",
+                                                            ftype[i].gline);
+          g_key_file_set_boolean (key_file, group_name, "mline",
+                                                            ftype[i].mline);
+          g_key_file_set_boolean (key_file, group_name, "nline",
+                                                            ftype[i].nline);
+          g_key_file_set_boolean (key_file, group_name, "uline",
+                                                            ftype[i].uline);
+          g_key_file_set_boolean (key_file, group_name, "vline",
+                                                            ftype[i].vline);
+          /* ja:キャラクターセット */
+          g_key_file_set_string (key_file, group_name, "charset",
                                                             ftype[i].charset);
-      /* ja:フォント */
-      if (ftype[i].font_name)
-        g_key_file_set_string (key_file, group_name, "font",
+          /* ja:フォント */
+          if (ftype[i].font_name)
+            g_key_file_set_string (key_file, group_name, "font",
                                                         ftype[i].font_name);
-      /* ja:色 */
-      list[(sizeof (GdkColor) * 12 + sizeof (gint) - 1) / sizeof (gint) - 1]
-                                                                        = 0;
-      g_memmove (list, ftype[i].color, sizeof (GdkColor) * 12);
-      g_key_file_set_integer_list (key_file, group_name, "color", list,
+          /* ja:色 */
+          list[(sizeof (GdkColor) * 12 + sizeof (gint) - 1)
+                                                    / sizeof (gint) - 1] = 0;
+          g_memmove (list, ftype[i].color, sizeof (GdkColor) * 12);
+          g_key_file_set_integer_list (key_file, group_name, "color", list,
                 (sizeof (GdkColor) * 12 + sizeof (gint) - 1) / sizeof (gint));
-      g_free (group_name);
-    }
-  while (group_name = g_strdup_printf ("filetype%04d", i++),
+          g_free (group_name);
+        }
+      while (group_name = g_strdup_printf ("filetype%04d", i++),
                                 g_key_file_has_group (key_file, group_name))
-    {
-      g_key_file_remove_group (key_file, group_name, NULL);
+        {
+          g_key_file_remove_group (key_file, group_name, NULL);
+          g_free (group_name);
+        }
       g_free (group_name);
-    }
-  g_free (group_name);
-  data = g_key_file_to_data (key_file, &length, NULL);
-  g_key_file_free (key_file);
-  if (data)
-    {
-      fileio_save (user_config, data, length);
-      g_free (data);
+      data = g_key_file_to_data (key_file, &length, NULL);
+      g_key_file_free (key_file);
+      if (data)
+        {
+          fileio_save (user_config, data, length);
+          g_free (data);
+        }
     }
 
   pango_font_description_free (system_font);
